#!/bin/sh
#
#Copyright (c) 2012, Novell, Inc.  All rights reserved.
#
# This script is used to reconfigure common proxy in ID swap migration scenarios
#

#This task should be executed immediately after ID SWAP Migration
#Don't delete or rename the proxy user in eDirectory
#Should be same as in OCS

#Tasks performed by the script
#1. Retrieve the current common proxy credenials(Proxy DN and Password) from OCS
#2. Retrieve the NCP Server DN from RootDSE. Extract the NCP Host name from NCP Server DN
#3. Form the NCP Server proxy name as OESCommonProxy_<NCPServerHostName>
#4. Rename the Proxy retrieved in step 1 to the proxy DN formed in step 3
#5. Update the common proxy credentials in OCS to point to the proxy DN renamed in step 4
#6. Identify the list of OES seervices installed to use common proxy
#7. Call move_to_common_proxy.sh to move the services from step 6 to use renamed proxy user

# All the logging goes to /var/opt/novell/log/proxymgmt/pxymgmt.log
# Errors and some important things are logged to stdout also. Done for migration framework


#Only root can execute this script
user_id=`id -u`
if test $user_id -ne 0
then 
        echo "mignwproxy.sh: Only root user can run this script"
        exit 1
fi

#File locations
PROXY_UTIL="/opt/novell/proxymgmt/bin"
PROXY_LOGFILE="/var/opt/novell/log/proxymgmt/pxymgmt.log"
SVC_PROXY_LOGFILE="/var/opt/novell/log/proxymgmt/pxylist.txt"
SVC_PROXY_TMPFILE="/var/opt/novell/log/proxymgmt/pxyuser.txt"
PROXY_CONF_FILE="/etc/opt/novell/proxymgmt/proxy_users.conf"

export LDAPTLS_REQCERT=never

#Help of the script
Usage()
{
	        echo ""
	        echo "Usage: mignwproxy.sh [options]"
	        echo "          -h                      Prints this summary"
	        echo "          -d                      LDAP Admin FDN"
	        echo "          -w                      LDAP Admin Password"
	        echo "          -i                      LDAP Server IP address"
	        echo "          -p                      LDAP Port"
		echo ""

		exit 1;
}

#Get the command line parameters
while getopts d:w:i:p:h o
do
        case "$o" in
	             d)LDAP_ADM="$OPTARG";;
	             w)LDAP_PWD="$OPTARG";;
	             i)LDAP_IP="$OPTARG";;
	             p)LDAP_PORT="$OPTARG";;
	             h)Usage;;
	           [?]) Usage;;
	esac
done

if test "$LDAP_ADM" = "" || test "$LDAP_IP" = "" || test "$LDAP_PORT" = "" 
then
	Usage
fi

#If Admin password is not passed through command line, it should be passed through environment variable OES_ADMIN_PWD
if test "$LDAP_PWD" = ""
then
	LDAP_PWD="$OES_ADMIN_PWD"
	if test "$LDAP_PWD" = ""
	then
		echo "LDAP Admin password is not provided"
		Usage
	fi
fi

echo "************************************************" >> $PROXY_LOGFILE
echo "`date`:[status] Starting Common Proxy Reconfiguration  Task" >> $PROXY_LOGFILE

#Retrieve current proxy credentials from OCS. If not present, common proxy is not in use. Nothing needs to be done
OLDCP_DN=`$PROXY_UTIL/cp_retrieve_proxy_cred username`
OCS_READUSR_RES=$?
if [ $OCS_READUSR_RES -ne 0 ]
then
        echo "Common Proxy not configured on the system. Could not retrieve common proxy DN from OCS. Nothing needs to be done: $OCS_READUSR_RES" >> $PROXY_LOGFILE
        echo "Common Proxy not configured on the system. Could not retrieve common proxy DN from OCS. Nothing needs to be done: $OCS_READUSR_RES" 
        exit $OCS_READUSR_RES
fi

CP_PWD=`$PROXY_UTIL/randpwd.sh` #Generate a random password
export OES_PROXY_DATA="$CP_PWD"
#
# Store the credentials in OCS store.

$PROXY_UTIL/cp_update_proxy_cred.sh "$OLDCP_DN" >> /dev/null 2>&1
STORE_IN_OCS=$?

if test $STORE_IN_OCS == 0
then
	echo "[Status] Common proxy password updated in common OCS store successfully" >> $PROXY_LOGFILE

fi
unset OES_PROXY_DATA

OLDCP_PWD=`$PROXY_UTIL/cp_retrieve_proxy_cred password `
OCS_READPWD_RES=$?
if [ $OCS_READPWD_RES -ne 0 ]
then
        echo "Common Proxy not configured on the system. Could not retrieve common proxy password from OCS. Nothing needs to be done: $OCS_READPWD_RES" >> $PROXY_LOGFILE
        echo "Common Proxy not configured on the system. Could not retrieve common proxy password from OCS. Nothing needs to be done: $OCS_READPWD_RES" 
        exit $OCS_READPWD_RES
fi

echo "Current Proxy DN: $OLDCP_DN" >> $PROXY_LOGFILE
OLDCPCN=`echo $OLDCP_DN | cut -d"," -f1`
echo "Current Proxy CN: $OLDCPCN" >> $PROXY_LOGFILE
DUPOLDCP=$OLDCP_DN
OLDCPCNTXT=`echo ${DUPOLDCP#*,}`
echo "Current Proxy Context: $OLDCPCNTXT" >> $PROXY_LOGFILE

#Now form the common proxy DN as per the NCP server name
#NCP Server name is retrieved from Root DSE
ROOTDSE=`env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapsearch -D "" -s base -H ldaps://$LDAP_IP:636 | grep dsaName`
LDAP_SEA_RES=$?
#Exit if we can't read from RootDSE
if test "$ROOTDSE" = ""
then
	echo "Failed to get the NCP Server name from From RootDSE. Run this script again manually: $LDAP_SEA_RES " >> $PROXY_LOGFILE
	echo "Failed to get the NCP Server name from From RootDSE. Run this script again manually: $LDAP_SEA_RES " 
	exit $LDAP_SEA_RES
fi

echo "DSA Entry from from RootDSE: $ROOTDSE" >> $PROXY_LOGFILE

#Extract the  NCP Server DN from RootDSE entry
NCPDN=`echo $ROOTDSE | cut -d" " -f2`
echo "NCP Server DN: $NCPDN" >> $PROXY_LOGFILE

#Extract the NCP Server CN
NCPCN=`echo $NCPDN | cut -d"," -f1`
echo "NCP Server CN: $NCPCN" >> $PROXY_LOGFILE

#Extract the NCP Server Context
NCPDUP=$NCPDN
NCPCNTXT=`echo ${NCPDUP#*,}`
echo "NCP Server Context: $NCPCNTXT" >> $PROXY_LOGFILE
 
#Get the Server Name by removing cn=
NCPHOSTNAME=`echo $NCPCN | cut -d"=" -f2`
echo "NCP Server Host Name: $NCPHOSTNAME" >> $PROXY_LOGFILE

#Form the Common Proxy CN for the host
NCPCPCN=`echo cn=OESCommonProxy_$NCPHOSTNAME`
echo "Common Proxy Name for the system should be: $NCPCPCN" >> $PROXY_LOGFILE

#If the current proxy is same the one we want, then exit
if [[ "$OLDCPCN" = "$NCPCPCN" ]]
then
	echo "Current Proxy is already configured with host name. Nothing needs to be done" >> $PROXY_LOGFILE
	echo "Current Proxy is already configured with host name. Nothing needs to be done" 
	exit 0;
else
	echo "Current Proxy user will be renamed as per the NCP Server host name" >> $PROXY_LOGFILE
fi

NCPCPDN=`echo $NCPCPCN,$OLDCPCNTXT`

#Rename the old proxy user as per the host name
echo "Common Proxy $OLDCP_DN is getting renamed to: $NCPCPDN" >> $PROXY_LOGFILE		
# -r option is added to below command to provide fix for bug#806982
# Enclosing the admin password in double quotes to support special characters. This is the fix for the bug #845803
env LDAPTLS_CACERT=/etc/opt/novell/certs/SSCert.pem /usr/bin/ldapmodrdn -D "$LDAP_ADM" -w "$LDAP_PWD" -H ldaps://$LDAP_IP:$LDAP_PORT "$OLDCP_DN" "$NCPCPCN" -r
CP_RENAME_RES=$?
if [ $CP_RENAME_RES  -ne 0 ]
then
	echo "Failed to rename the proxy user $OLDCPCN to $NCPCPCN: $CP_RENAME_RES" >> $PROXY_LOGFILE
	echo "Failed to rename the proxy user $OLDCPCN to $NCPCPCN: $CP_RENAME_RES" 
	if [ $CP_RENAME_RES -eq 68 ]
	then
		echo "$NCPCPCN already exists in eDiectory. Will be recreated by move_to_common_proxy.sh" >> $PROXY_LOGFILE
		echo "$NCPCPCN already exists in eDiectory. Will be recreated by move_to_common_proxy.sh" 
		CP_RENAME_RES=0
	fi

	if [ $CP_RENAME_RES -eq 32 ]
	then
		echo "$OLDCP_DN is not found in eDirectory. But, credentials are found in OCS" >> $PROXY_LOGFILE
		echo "$OLDCP_DN is not found in eDirectory. But, credentials are found in OCS" 
		echo "Delete the common proxy credentials from OCS and run move_to_common_proxy.sh for reconfiguring common proxy" >> $PROXY_LOGFILE
		echo "Delete the common proxy credentials from OCS and run move_to_common_proxy.sh for reconfiguring common proxy"
		exit $CP_RENAME_RES
	fi
	if [ $CP_RENAME_RES -ne 0 ]
	then
		echo "Run this script again" >> $PROXY_LOGFILE
		echo "Run this script again"
		exit $CP_RENAME_RES
	fi
fi
export LDAPTLS_REQCERT=never
export OES_DATA="$LDAP_PWD"
echo "Resetting password in eDirectory"
echo "Resetting password in eDirectory" >> $PROXY_LOGFILE
/opt/novell/proxymgmt/bin/change_pwd_upgrade.sh -u "$NCPCPDN" -p "$LDAP_ADM"

STORE_IN_OCS=$?
if test $STORE_IN_OCS == 0
then
                echo "common proxy credentials got stored in eDirectory successfully" >> $PROXY_LOGFILE
else
		echo "Error while storing proxy credentials in eDirectory: $STORE_IN_OCS " >> $PROXY_LOGFILE
                exit $STORE_IN_OCS
fi

unset LDAPTLS_REQCERT

OLDCP_PWD=`$PROXY_UTIL/cp_retrieve_proxy_cred password`

echo "Common Proxy is renamed successfully " >> $PROXY_LOGFILE

#Update proxy user name in OCS

export OES_DATA="$LDAP_PWD"
export OES_PROXY_DATA="$OLDCP_PWD"

#Store the OCS credentials in OCS
/opt/novell/proxymgmt/bin/cp_update_proxy_cred.sh "$NCPCPDN" >> /dev/null 2>&1
STORE_IN_OCS=$?
if test $STORE_IN_OCS == 0
then
                echo "common proxy credentials got stored in OCS successfully" >> $PROXY_LOGFILE
else
		echo "Error while storing proxy credentials in OCS. Run the tool again: $STORE_IN_OCS " >> $PROXY_LOGFILE
		echo "Error while storing proxy credentials in OCS. Run the tool again: $STORE_IN_OCS " 
		exit $STORE_IN_OCS
fi

#Remove old proxy user name from file /etc/opt/novell/proxymgmt/proxy_users.conf
#This fix is provided for bug #810204
echo "Deleting old common proxy user $OLDCP_DN from $PROXY_CONF_FILE"
# Delete old common proxy user only if its lenght is non-zero and it is not empty
if [ -n "$OLDCP_DN" ] && [ ! -z "$OLDCP_DN" ]
then
  sed -i "/${OLDCP_DN}/d" $PROXY_CONF_FILE
fi

echo "Retrieving list of services installed with common proxy" >> $PROXY_LOGFILE

#Get list of services installed to use common proxy

#AFP
if [ -f "/opt/novell/afptcpd/bin/afp_retrieve_proxy_cred.sh" ];
then	
	/opt/novell/afptcpd/bin/afp_retrieve_proxy_cred.sh username $SVC_PROXY_TMPFILE
	if [ -f $SVC_PROXY_TMPFILE ];
	then
		read SRVC_PROXY < $SVC_PROXY_TMPFILE
		if [ "$SRVC_PROXY" = "$OLDCP_DN" ]
		then
			echo "AFP is installed with Common Proxy. Will be reconfigured to use new common proxy " >> $PROXY_LOGFILE
			SRVNAME="novell-afp"
			CPSERVICES=`echo $CPSERVICES,$SRVNAME`
		fi
		rm $SVC_PROXY_TMPFILE
	fi
fi

#CIFS
if [ -f "/opt/novell/cifs/bin/cifs_retrieve_proxy_cred.sh" ];
then	
	/opt/novell/cifs/bin/cifs_retrieve_proxy_cred.sh username $SVC_PROXY_TMPFILE
	if [ -f $SVC_PROXY_TMPFILE ];
	then
		read SRVC_PROXY < $SVC_PROXY_TMPFILE
		if [ "$SRVC_PROXY" = "$OLDCP_DN" ]
		then
			echo "CIFS is installed with Common Proxy. Will be reconfigured to use new common proxy " >> $PROXY_LOGFILE
			SRVNAME="novell-cifs"
			CPSERVICES=`echo $CPSERVICES,$SRVNAME`
		fi
		rm $SVC_PROXY_TMPFILE
	fi
fi


#DNS Proxy User 
if [ -f "/opt/novell/named/bin/dns_retrieve_proxy_cred" ];
then	
	/opt/novell/named/bin/dns_retrieve_proxy_cred username $SVC_PROXY_TMPFILE			
	if [ -f $SVC_PROXY_TMPFILE ];
	then
		read SRVC_PROXY < $SVC_PROXY_TMPFILE
		rm $SVC_PROXY_TMPFILE
		if [ "$SRVC_PROXY" = "$OLDCP_DN" ];
		then
			echo "DNS is installed with Common Proxy. Will be reconfigured to use new common proxy" >> $PROXY_LOGFILE 
			SRVNAME="novell-dns"
			CPSERVICES=`echo $CPSERVICES,$SRVNAME`
		fi
	fi
fi

#DHCP Proxy User 
if [ -f "/opt/novell/dhcp/bin/dhcp_retrieve_proxy_cred" ];
then	
	/opt/novell/dhcp/bin/dhcp_retrieve_proxy_cred username $SVC_PROXY_TMPFILE			
	if [ -f $SVC_PROXY_TMPFILE ];
	then
		read SRVC_PROXY < $SVC_PROXY_TMPFILE
		rm $SVC_PROXY_TMPFILE
		if [ "$SRVC_PROXY" = "$OLDCP_DN" ];
		then
			echo "DHCP is installed with Common Proxy. Will be reconfigured to use new common proxy" >> $PROXY_LOGFILE
			SRVNAME="novell-dhcp"
			CPSERVICES=`echo $CPSERVICES,$SRVNAME`
		fi
	fi
fi

#NetStorage Proxy User 
if [ -f "/opt/novell/netstorage/bin/ns_retrieve_proxy_cred" ];
then	
	/opt/novell/netstorage/bin/ns_retrieve_proxy_cred username $SVC_PROXY_TMPFILE			
	if [ -f $SVC_PROXY_TMPFILE ];
	then
		read SRVC_PROXY < $SVC_PROXY_TMPFILE
		rm $SVC_PROXY_TMPFILE
		if [ "$SRVC_PROXY" = "$OLDCP_DN" ];
		then
			echo "NetStorage is installed with Common Proxy. Will be reconfigured to use new common proxy" >> $PROXY_LOGFILE
			SRVNAME="novell-netstorage"
			CPSERVICES=`echo $CPSERVICES,$SRVNAME`
		fi
	fi
fi

#NCS Proxy User 
if [ -f "/opt/novell/ncs/bin/ncs_retrieve_proxy_cred" ];
then	
	/opt/novell/ncs/bin/ncs_retrieve_proxy_cred username $SVC_PROXY_TMPFILE			
	if [ -f $SVC_PROXY_TMPFILE ];
	then
		read SRVC_PROXY < $SVC_PROXY_TMPFILE
		rm $SVC_PROXY_TMPFILE
		if [ "$SRVC_PROXY" = "$OLDCP_DN" ];
		then
			echo "Novell Cluster Services is installed with Common Proxy. Will be reconfigured to use new common proxy." >> $PROXY_LOGFILE
			SRVNAME="novell-ncs"
			CPSERVICES=`echo $CPSERVICES,$SRVNAME`
		fi
	fi
fi

#LUM Proxy User 
if [ -f "/usr/bin/lum_retrieve_proxy_cred" ];
then	
	/usr/bin/lum_retrieve_proxy_cred username $SVC_PROXY_TMPFILE		
	if [ -f $SVC_PROXY_TMPFILE ];
	then
		read SRVC_PROXY < $SVC_PROXY_TMPFILE
		rm $SVC_PROXY_TMPFILE
		if [ "$SRVC_PROXY" = "$OLDCP_DN" ];
		then
			echo "LUM is installed with Common Proxy. Will be reconfigured to use new common proxy" >> $PROXY_LOGFILE
			SRVNAME="novell-lum"
			CPSERVICES=`echo $CPSERVICES,$SRVNAME`
		fi
	fi
fi

if test "$CPSERVICES" = ""
then
	echo "No service is using $OLDCP_DN. No service is reconfigured " >> $PROXY_LOGFILE
	echo "No service is using $OLDCP_DN. No service is reconfigured "
	exit 0
fi
CPSERVICES=`echo "$CPSERVICES" | sed 's/^,//'`
echo "Services getting reconfigured to use new common proxy: $CPSERVICES" >> $PROXY_LOGFILE
echo "Services getting reconfigured to use new common proxy: $CPSERVICES" 

/opt/novell/proxymgmt/bin/move_to_common_proxy.sh -d "$LDAP_ADM" -w "$LDAP_PWD" -i $LDAP_IP -p $LDAP_PORT -s $CPSERVICES

echo "Reconfiguring the system for common proxy is completed" >> $PROXY_LOGFILE
echo "Reconfiguring the system for common proxy is completed"
echo "Refer to $PROXY_LOGFILE for more details" 
unset LDAPTLS_REQCERT

